#ifndef NPSTAT_BOUNDARYMETHOD_HH_
#define NPSTAT_BOUNDARYMETHOD_HH_

/*!
// \file BoundaryMethod.hh
//
// \brief Enumeration of typical boundary handling methods for LOrPE
//
// Author: I. Volobouev
//
// March 2014
*/

#include <string>

namespace npstat {
    /**
    // BM_TRUNCATE: use simple truncation of the weight function,
    //              with subsequent renormalization
    //
    // BM_STRETCH:  stretch the weight function near the boundary
    //
    // BM_FOLD:     fold the weight function back into the support interval
    //
    // BM_CONSTSQ:  stretch the weight function so that, when it is
    //              normalized, the integral of its square remains constant
    //
    // BM_FOLDSQ:   combine folding and stretching so that the integral of
    //              weight squared remains constant
    */
    enum BoundaryMethod
    {
        BM_TRUNCATE = 0,
        BM_STRETCH,
        BM_FOLD,
        BM_CONSTSQ,
        BM_FOLDSQ
    };

    /** Enums corresponding to method names */
    BoundaryMethod parseBoundaryMethod(const char* methodName);

    /** Method names corresponding to enums */
    const char* boundaryMethodName(BoundaryMethod m);

    /** All valid method names for use in error messages, etc */
    std::string validBoundaryMethodNames();
}

#endif // NPSTAT_BOUNDARYMETHOD_HH_
