#ifndef NPSTAT_INTERPOLATEDDISTRO1D1P_HH_
#define NPSTAT_INTERPOLATEDDISTRO1D1P_HH_

/*!
// \file InterpolatedDistro1D1P.hh
//
// \brief Interpolation of 1-d distributions as a function of one parameter
//
// Author: I. Volobouev
//
// December 2014
*/

#include "geners/CPP11_shared_ptr.hh"

#include "npstat/nm/GridAxis.hh"
#include "npstat/stat/InterpolatedDistribution1D.hh"

namespace npstat {
    class InterpolatedDistro1D1P : public InterpolatedDistribution1D
    {
    public:
        InterpolatedDistro1D1P(
            const GridAxis& axis,
            const std::vector<CPP11_shared_ptr<const AbsDistribution1D> >& distros,
            double initialParameterValue = 0.0);

        inline virtual ~InterpolatedDistro1D1P() {}

        /** Set the value of the parameter */
        void setParameter(double value);

        /** Get the value of the parameter */
        inline double getParameter() const {return param_;}

        /** "Virtual copy constructor" */
        inline virtual InterpolatedDistro1D1P* clone() const
            {return new InterpolatedDistro1D1P(*this);}

        //@{
        /** Prototype needed for I/O */
        virtual inline gs::ClassId classId() const {return gs::ClassId(*this);}
        virtual bool write(std::ostream&) const;
        //@}

        static inline const char* classname()
            {return "npstat::InterpolatedDistro1D1P";}
        static inline unsigned version() {return 1;}
        static InterpolatedDistro1D1P* read(const gs::ClassId& id,
                                            std::istream& in);
    protected:
        virtual bool isEqual(const AbsDistribution1D&) const;

    private:
        InterpolatedDistro1D1P();

        GridAxis axis_;
        std::vector<CPP11_shared_ptr<const AbsDistribution1D> > distros_;
        double param_;
    };
}

#endif // NPSTAT_INTERPOLATEDDISTRO1D1P_HH_
