#ifndef NPSTAT_VERTICALLYINTERPOLATEDDISTRO1D1P_HH_
#define NPSTAT_VERTICALLYINTERPOLATEDDISTRO1D1P_HH_

/*!
// \file VerticallyInterpolatedDistro1D1P.hh
//
// \brief 1-d distribution vertically interpolated in a transformed space
//
// Author: I. Volobouev
//
// April 2015
*/

#include "geners/CPP11_shared_ptr.hh"

#include "npstat/nm/GridAxis.hh"
#include "npstat/stat/AbsDistribution1D.hh"
#include "npstat/stat/AbsDistributionTransform1D.hh"

namespace npstat {
    class VerticallyInterpolatedDistro1D1P : public AbsDistribution1D
    {
    public:
        /**
        //  The transform specified in the constructor must have at least
        //  one parameter. The 0th parameter of the transform will be used
        //  by this function.
        */
        VerticallyInterpolatedDistro1D1P(
            const GridAxis& axis,
            const std::vector<CPP11_shared_ptr<const AbsDistribution1D> >& distros,
            const AbsDistributionTransform1D& transform);

        VerticallyInterpolatedDistro1D1P(const VerticallyInterpolatedDistro1D1P& r);
        VerticallyInterpolatedDistro1D1P& operator=(
            const VerticallyInterpolatedDistro1D1P& r);

        inline virtual ~VerticallyInterpolatedDistro1D1P() {delete t_;}

        inline AbsDistributionTransform1D& getTransform() {return *t_;}
        inline const AbsDistributionTransform1D& getTransform() const 
            {return *t_;}

        /** Set the value of the parameter */
        inline void setParameter(const double value)
            {t_->setParameter(0U, value); changed_ = true;}

        /** Get the value of the parameter */
        inline double getParameter() const {return t_->getParameter(0U);}

        /** "Virtual copy constructor" */
        inline virtual VerticallyInterpolatedDistro1D1P* clone() const
            {return new VerticallyInterpolatedDistro1D1P(*this);}

        /** Probability density */
        virtual double density(double x) const;

        /** Cumulative distribution function */
        virtual double cdf(double x) const;

        /** 1 - cdf, implementations should avoid subtractive cancellation */
        virtual double exceedance(double x) const;

        /** The quantile function */
        virtual double quantile(double x) const;

        //@{
        /** Prototype needed for I/O */
        virtual inline gs::ClassId classId() const {return gs::ClassId(*this);}
        virtual bool write(std::ostream&) const;
        //@}

        static inline const char* classname()
            {return "npstat::VerticallyInterpolatedDistro1D1P";}
        static inline unsigned version() {return 1;}
        static VerticallyInterpolatedDistro1D1P* read(const gs::ClassId& id,
                                                      std::istream& in);
    protected:
        virtual bool isEqual(const AbsDistribution1D&) const;

    private:
        VerticallyInterpolatedDistro1D1P();
        void calcWeights() const;

        double cdf_in_y(double y) const;
        double quantile_in_y(double y) const;

        GridAxis axis_;
        std::vector<CPP11_shared_ptr<const AbsDistribution1D> > distros_;
        AbsDistributionTransform1D* t_;

        mutable double wleft_;
        mutable double wright_;
        mutable unsigned cell_;
        mutable bool changed_;
    };
}

#endif // NPSTAT_VERTICALLYINTERPOLATEDDISTRO1D1P_HH_
